import { Injectable } from '@angular/core';

import { Observable } from 'rxjs';
import { map, take, takeWhile } from 'rxjs/operators';

import { ProtocolService } from '@ucap-webmessenger/protocol';
import {
  BuddyRequest,
  BuddyResponse,
  BuddyDataResponse,
  encodeBuddy,
  decodeBuddyData,
  decodeBuddy,
  decodeBuddyDataDetail
} from '../models/buddy';
import {
  SVC_TYPE_SYNC_BUDDY,
  SSVC_TYPE_SYNC_BUDDY_REQ,
  SSVC_TYPE_SYNC_BUDDY_RES,
  SSVC_TYPE_SYNC_BUDDY_DATA,
  SSVC_TYPE_SYNC_BUDDY2_REQ,
  SSVC_TYPE_SYNC_BUDDY2_RES,
  SSVC_TYPE_SYNC_BUDDY2_DATA,
  SVC_TYPE_SYNC_GROUP,
  SSVC_TYPE_SYNC_GROUP_REQ,
  SSVC_TYPE_SYNC_GROUP_RES,
  SSVC_TYPE_SYNC_GROUP_DATA,
  SSVC_TYPE_SYNC_GROUP_REQ2,
  SSVC_TYPE_SYNC_GROUP_RES2,
  SSVC_TYPE_SYNC_GROUP_DATA2,
  SVC_TYPE_SYNC_ROOM,
  SSVC_TYPE_SYNC_ROOM_REQ,
  SSVC_TYPE_SYNC_ROOM_RES,
  SSVC_TYPE_SYNC_ROOM_DATA,
  SSVC_TYPE_SYNC_ROOM_USER,
  SSVC_TYPE_SYNC_ROOM_USER2,
  SSVC_TYPE_SYNC_ROOM2_REQ,
  SSVC_TYPE_SYNC_ROOM2_RES,
  SSVC_TYPE_SYNC_ROOM2_DATA,
  SSVC_TYPE_SYNC_ROOM2_USER,
  SSVC_TYPE_SYNC_ROOM2_USER2
} from '../types/service';
import {
  GroupRequest,
  GroupResponse,
  GroupDataResponse,
  encodeGroup,
  decodeGroupDataDetail,
  decodeGroup,
  decodeGroupData,
  GroupDataDetailResponse
} from '../models/group';
import {
  RoomRequest,
  RoomResponse,
  encodeRoom,
  decodeRoom,
  RoomUserDataResponse,
  RoomUserDataDetailResponse,
  RoomDataResponse,
  decodeRoomData,
  decodeRoomUserData,
  decodeRoomUserDataDetail
} from '../models/room';

@Injectable({
  providedIn: 'root'
})
export class SyncProtocolService {
  constructor(private protocolService: ProtocolService) {}

  public buddy(
    req: BuddyRequest
  ): Observable<BuddyResponse | BuddyDataResponse> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_BUDDY, SSVC_TYPE_SYNC_BUDDY_REQ, ...encodeBuddy(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_BUDDY_RES !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_BUDDY_DATA === res.subServiceType) {
            return decodeBuddyData(res);
          }

          return decodeBuddy(res);
        })
      );
  }

  public buddy2(
    req: BuddyRequest
  ): Observable<BuddyResponse | BuddyDataResponse> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_BUDDY, SSVC_TYPE_SYNC_BUDDY2_REQ, ...encodeBuddy(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_BUDDY2_RES !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_BUDDY2_DATA === res.subServiceType) {
            return decodeBuddyDataDetail(res);
          }

          return decodeBuddy(res);
        })
      );
  }

  public syncGroup(
    req: GroupRequest
  ): Observable<GroupResponse | GroupDataResponse> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_GROUP, SSVC_TYPE_SYNC_GROUP_REQ, ...encodeGroup(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_GROUP_RES !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_GROUP_DATA === res.subServiceType) {
            return decodeGroupData(res);
          }

          return decodeGroup(res);
        })
      );
  }

  public syncGroup2(
    req: GroupRequest
  ): Observable<GroupResponse | GroupDataDetailResponse> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_GROUP, SSVC_TYPE_SYNC_GROUP_REQ2, ...encodeGroup(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_GROUP_RES2 !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_GROUP_DATA2 === res.subServiceType) {
            return decodeGroupDataDetail(res);
          }

          return decodeGroup(res);
        })
      );
  }

  public syncRoom(
    req: RoomRequest
  ): Observable<
    | RoomResponse
    | RoomDataResponse
    | RoomUserDataResponse
    | RoomUserDataDetailResponse
  > {
    return this.protocolService
      .call(SVC_TYPE_SYNC_ROOM, SSVC_TYPE_SYNC_ROOM_REQ, ...encodeRoom(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_ROOM_RES !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_ROOM_DATA === res.subServiceType) {
            return decodeRoomData(res);
          } else if (SSVC_TYPE_SYNC_ROOM_USER === res.subServiceType) {
            return decodeRoomUserData(res);
          } else if (SSVC_TYPE_SYNC_ROOM_USER2 === res.subServiceType) {
            return decodeRoomUserDataDetail(res);
          }

          return decodeRoom(res);
        })
      );
  }

  public syncRoom2(
    req: RoomRequest
  ): Observable<
    | RoomResponse
    | RoomDataResponse
    | RoomUserDataResponse
    | RoomUserDataDetailResponse
  > {
    return this.protocolService
      .call(SVC_TYPE_SYNC_ROOM, SSVC_TYPE_SYNC_ROOM2_REQ, ...encodeRoom(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_ROOM2_RES !== res.subServiceType),
        map(res => {
          if (SSVC_TYPE_SYNC_ROOM2_DATA === res.subServiceType) {
            return decodeRoomData(res);
          } else if (SSVC_TYPE_SYNC_ROOM2_USER === res.subServiceType) {
            return decodeRoomUserData(res);
          } else if (SSVC_TYPE_SYNC_ROOM2_USER2 === res.subServiceType) {
            return decodeRoomUserDataDetail(res);
          }

          return decodeRoom(res);
        })
      );
  }
}
